//
//  CircularGauge.m
//  HA7 logger mini
//  Copyright(C)2012 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2012.02.08 v0.0  First cording
//    2012.02.12 v0.2  Commit
//

#import "CircularGauge.h"

@implementation CircularGauge
@synthesize val;
@synthesize val_l;
@synthesize val_h;
@synthesize val_w;
@synthesize val_c;
@synthesize val_peak1;
@synthesize val_peak2;
@synthesize val_low;
@synthesize val_high;
@synthesize val_warning;
@synthesize val_critical;

- (void)awakeFromNib {
	val_h = [val_high floatValue];
	val_l = [val_low floatValue];
	val_w = [val_warning floatValue];
	val_c = [val_critical floatValue];
	val   = 0;
	val_peak1 = self.val;
	val_peak2 = self.val;
}

- (void)drawRect:(NSRect)rect {
	float cx,cy;
	float r1,r2;
	float x1,y1;
	float x2,y2;
	float ratio;
	float rad;
		
	cx = NSMidX(rect);
	cy = NSHeight(rect) - cx;
	r1 = NSWidth(rect) * 0.5;
	r2 = NSWidth(rect) * 0.1;

	// Maximum peak needle
	ratio = (val_peak1 - val_l) / (val_h - val_l);
	rad = pi * -ratio + pi;
	x1 = cos(rad) * r1 + cx;
	y1 = sin(rad) * r1 + cy;
	x2 = cos(rad) * r2 + cx;
	y2 = sin(rad) * r2 + cy;
	[[NSColor grayColor] set];
	[NSBezierPath setLineWidth:1.0];
	[NSBezierPath strokeLineFromPoint:NSMakePoint(x1,y1) toPoint:NSMakePoint(x2,y2)];

	// Minimum peak needle
	ratio = (val_peak2 - val_l) / (val_h - val_l);
	rad = pi * -ratio + pi;
	x1 = cos(rad) * r1 + cx;
	y1 = sin(rad) * r1 + cy;
	x2 = cos(rad) * r2 + cx;
	y2 = sin(rad) * r2 + cy;
	[[NSColor grayColor] set];
	[NSBezierPath setLineWidth:1.0];
	[NSBezierPath strokeLineFromPoint:NSMakePoint(x1,y1) toPoint:NSMakePoint(x2,y2)];
	
	// Main needle
	ratio = (val - val_l) / (val_h - val_l);
	rad = pi * -ratio + pi;
	x1 = cos(rad) * r1 + cx;
	y1 = sin(rad) * r1 + cy;
	x2 = cos(rad) * r2 + cx;
	y2 = sin(rad) * r2 + cy;
	if (val > val_c) {
		[[NSColor redColor] set];
	} else if (val > val_w) {
		[[NSColor yellowColor] set];
	} else {
		[[NSColor greenColor] set];
	}
	[NSBezierPath setLineWidth:5.0];
	[NSBezierPath strokeLineFromPoint:NSMakePoint(x1,y1) toPoint:NSMakePoint(x2,y2)];

}

- (IBAction)setValue:(id)sender {
    val = [sender floatValue];
    [self setNeedsDisplay:YES];
	
	if (val_peak1 < val) {
		val_peak1 = val;
	}
	if (val_peak2 > val) {
		val_peak2 = val;
	}
}

- (IBAction)setIntValue:(int)sender {
    val = sender;
    [self setNeedsDisplay:YES];
	
	if (val_peak1 < val) {
		val_peak1 = val;
	}
	if (val_peak2 > val) {
		val_peak2 = val;
	}
}

- (IBAction)setFloatValue:(float)sender {
    val = sender;
    [self setNeedsDisplay:YES];

	if (val_peak1 < val) {
		val_peak1 = val;
	}
	if (val_peak2 > val) {
		val_peak2 = val;
	}
}

- (IBAction)setPeakValue:(id)sender {
    val_peak1 = val;
    val_peak2 = val;
    [self setNeedsDisplay:YES];
}

- (void)mouseDown:(NSEvent *)theEvent {
    val_peak1 = val;
    val_peak2 = val;
    [self setNeedsDisplay:YES];	
}

@end
