//
//  HA7 logger mini_AppDelegate.m
//  HA7 logger mini
//  Copyright(C)2012 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//	Description
//		Simple vehicle data logger for "HONDA GBD-HA7".
//		Needs the special USB device.
//		That is a FTDI FT232R(USB-Serial converter) and ISO9141-2 K-Line interface circuit.
//
//	Usage
//		Install FTDI D2XX driver software.
//		Connect a PC to USB adapter.
//		Connect a USB adapter to vehicle's OBD2 socket.
//		Start up this program.
//
//	Environment:
//		Runtime:
//			Mac OS X 10.6 -
//		Development:
//			Mac OS X 10.6.8
//			Xcode 3.2.6
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2012.01.02 v0.0  First cording
//    2012.01.12 v0.1  Commit
//    2012.02.12 v0.2  Add efficiency items, and change UI
//

#import "HA7 logger mini_AppDelegate.h"

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include "iso9141_io.h"

#define INTVL 2.0						// ADD 2012.2.10 [sec]
#define C_VOL 0.00066					// ADD 2012.2.10 Cylnder volume [m^3]
#define E_CYC 2.0						// ADD 2012.2.10 Engine output shaft rotation / Cycle

FILE	*fp = NULL;
double	dist_s = 0.0;					// ADD 2012.2.10
double	fuse_s = 0.0;					// ADD 2012.2.10
double	runtm  = 0.0;					// ADD 2012.2.10

@implementation HA7_logger_mini_AppDelegate
@synthesize window;


- (void)applicationDidFinishLaunching:(NSNotification *)aNotification {
	//
	// Start collection					// ADD START 2012.2.10
	//
	updateTimer = [NSTimer scheduledTimerWithTimeInterval:2
												   target:self
												 selector:@selector(GetData:)
												 userInfo:nil
												  repeats:YES];
										// ADD END 2012.2.10
}

- (IBAction)GetData:(id)sender
{
	int				rc;
	size_t			len;
	unsigned char	buf[16];
	int				i;
	
	//
	// Scan values
	//
	for(;;) {
		unsigned char	pid[] = {0x05, 0x06, 0x07, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f, 0x11, 0x14};
		int		ect;
		float	shrtft1;
		float	longft1;
		int		map;
		int		rpm;
		int		vss;
		int		sparkadv;
		int		iat;
		float	tp;
		float	o2s11;
		float	shrtft11;
		float	af;							// ADD 2012.2.10
		float	ff;							// ADD 2012.2.10
		float	r;							// ADD 2012.2.10
		float	v;							// ADD 2012.2.10
		float	atm;						// ADD 2012.2.10
		float	dist;						// ADD 2012.2.10
		float	fuse;						// ADD 2012.2.10
		float	eff;						// ADD 2012.2.10
		float	eff2;						// ADD 2012.2.10
		
		//									// ADD START 2012.2.10
		// Preprocess
		//
		{
			
			//
			// Logfile
			//
			if ([logf intValue]) {
				if (fp == NULL) {
					char	fname[256];
					
					sprintf(fname, "%s/Desktop/HA7.csv", getenv("HOME"));
					if ((fp = fopen(fname, "a")) == NULL) {
						[msg setStringValue:@"Log file open error"];
						break;
					}
					fprintf(fp, "\"DateTime\",\"ect[˚C]\",\"shrtft1[%%]\",\"longft1[%%]\",\"map[kPa]\",\"rpm[rpm]\",\"vss[km/h]\",\"sparkadv[˚]\",\"iat[˚C]\",\"tp[%%]\",\"o2s11[V]\",\"shrtft11[%%]\"\n");
				}
			} else {
				if (fp != NULL) {
					fclose(fp);
					fp = NULL;
				}
			}

			//
			// Vehcle communication
			//
			if ([cnct intValue]) {
				//
				// Connect to USB -> FT232R
				//
				if ([cnst intValue] < 2) {
					char	dev[64];
					
					[msg setStringValue:@"Connect OBD2/ISO9141-2 via USB UART FT232R"];
					if ((rc = iso9141_open(dev)) != 0) {
						[msg setStringValue:@"Not found FTDI FT232R USB device"];
						break;
					}
					[cnst setIntValue:2];
				}
				
				//
				// Connect to FT232R -> OBD2 -> ISO9141-2
				//
				if ([cnst intValue] < 3) {
					unsigned char	keyword[2];

					[msg setStringValue:@"Connect to vehicle..."];
					if ((rc = iso9141_init(keyword)) != 0) {
						[msg setStringValue:@"Not response a vehicle"];
						break;
					}
					if ((keyword[0] == 0x08) && (keyword[1] == 0x08)) {
						[msg setStringValue:@"Connected: Protocol is ISO-9141-2"];
					} else if ((keyword[0] == 0x94) && (keyword[1] == 0x94)) {
						[msg setStringValue:@"Connected: Protocol is ISO-9141-2(0x9494)"];
					} else {
						[msg setStringValue:@"Error: Unknown protocol"];
						break;
					}
					[cnst setIntValue:3];
				}

			} else {
				//
				// Disconnect
				//
				if ([cnst intValue] > 1) {
					iso9141_close();
					[msg setStringValue:@"Disconnected"];
					[cnst setIntValue:1];
				}
				break;
			}
			
		}										// ADD END 2012.2.10
		
		
		//
		// Get data
		//
		for(i = 0; i < sizeof(pid); i++) {
			
			buf[0] = 0x01;		// Request current powertrain diagnostic data request SID
			buf[1] = pid[i];
			if ((rc = iso9141_send_msg(buf, 2)) != 0) {
				break;
			}
			if ((rc = iso9141_recv_response(buf, &len)) != 0) {
				break;
			}
			
			switch(i){
				case 0:
					ect = buf[2] - 40;
					break;
				case 1:
					shrtft1 = (buf[2] - 128) * 100 / 128.0;
					break;
				case 2:
					longft1 = (buf[2] - 128) * 100 / 128.0;
					break;
				case 3:
					map = buf[2];
					break;
				case 4:
					rpm = (buf[2] * 256 + buf[3]) / 4;
					break;
				case 5:
					vss = buf[2];
					break;
				case 6:
					sparkadv = (buf[2] - 128) / 2;
					break;
				case 7:
					iat = buf[2] - 40;
					break;
				case 8:
					tp = buf[2] * 100 / 255.0;
					break;
				case 9:
					o2s11 = (float)buf[2] * 0.005;
					shrtft11 = (buf[3] - 128) * 100 / 128.0;
					break;
				default:
					break;
			}
			
		}
												// ADD START 2012.2.10
		if (rc != 0) {
			iso9141_close();
			[msg setStringValue:@"Communication error."];
			[cnst setIntValue:1];
			break;
		}
												// ADD END 2012.2.10
		
		//
		// Display
		//
		[ect_v setIntValue:ect];
		[ect_g setIntValue:ect];
		
		[shrtft1_v setFloatValue:shrtft1];
		[shrtft1_g setFloatValue:shrtft1];
		
		[longft1_v setFloatValue:longft1];
		[longft1_g setFloatValue:longft1];
		
		[map_v setIntValue:map];
		[map_g setIntValue:map];
		
		[rpm_v setIntValue:rpm];
		[rpm_g setIntValue:rpm];
		
		[vss_v setIntValue:vss];
		[vss_g setIntValue:vss];
		
		[sparkadv_v setIntValue:sparkadv];
		[sparkadv_g setIntValue:sparkadv];
		
		[iat_v setIntValue:iat];
		[iat_g setIntValue:iat];
		
		[tp_v setFloatValue:tp];
		[tp_g setFloatValue:tp];
		
		[o2s11_v setFloatValue:o2s11];
		[o2s11_g setFloatValue:o2s11 * 1000.0];
		
		[shrtft11_v setFloatValue:shrtft11];
		[shrtft11_g setFloatValue:shrtft11];

		//
		// Logging
		//
		if (fp != NULL) {
			time_t	t;
			struct tm *tm;
			
			time(&t);
			tm = localtime(&t);
			fprintf(fp, "\"%d-%02d-%02d %02d:%02d:%02d\",",
					tm->tm_year+1900, tm->tm_mon+1, tm->tm_mday, tm->tm_hour, tm->tm_min, tm->tm_sec);
			fprintf(fp, "%d,%f,%f,%d,%d,%d,%d,%d,%f,%f,%f\n",
					ect, shrtft1, longft1, map, rpm, vss, sparkadv, iat, tp, o2s11, shrtft11);
		}

		//
		// Addition information							// ADD START 2012.2.10
		//
		
		// Calcurate
		atm = map * 1000.0 / 101325.0;					// Air pressure [atm]
		r = (1.293 * atm) / (1 + 0.00367 * iat);		// Air density [kg/m^3]
		v = (float)rpm / 60.0 / E_CYC * C_VOL;			// Air volume [m^3/sec]
		af = r * v * 1000.0;							// Air flow on intake manifold [g/sec]

		ff = af / 14.7 * (shrtft1 / 100.0 + 1.0);		// Fuel flow on intake manifold [g/sec] 
		
		dist = (vss / (60.0 * 60.0) * INTVL);			// Interval distance [km]
		fuse = (ff / 0.783 / 1000.0 * INTVL);			// Interval used fuel [l]

		// Estimate
		dist_s += dist;									// 
		fuse_s += fuse;									//
		runtm  += INTVL;								//

		// Eefficient
		if (dist < 0.0001) {							// 10[cm]
			eff = 0.0;									//
		} else if (fuse > 0.00001) {					// 0.01[ml]
			eff = dist / fuse;							// [km/l]
		} else {										//
			eff = 999;									//
		}												//
		if (dist_s < 0.001) {							// 1[m]
			eff2 = 0;									//
		} else if (fuse_s > 0.001) {					// 1[ml]
			eff2 = dist_s / fuse_s;						//
		} else {										//
			eff2 = 999;									//
		}												//
		
		// Display
		[af_v setFloatValue:af];						// [g/sec]
		[af_g setFloatValue:af];						// [g/sec]
		[ff_v setFloatValue:ff];						// [g/sec]
		[ff_g setFloatValue:ff];						// [g/sec]
		[dist_v setFloatValue:dist_s];					// [km]
		[dist_g setFloatValue:dist_s];					// [km]
		[fuse_v setFloatValue:fuse_s];					// [l]
		[fuse_g setFloatValue:fuse_s];					// [l]
		[eff_v setFloatValue:eff];						// [km/l]
		[eff_g setFloatValue:eff];						// [km/l]
		[eff2_v setFloatValue:eff2];					// [km/l]
		[eff2_g setFloatValue:eff2];					// [km/l]
														// ADD END 2012.2.10

		break;
	}
	
}


- (void)windowWillClose:(NSNotification *)aNotification
{
	iso9141_close();									// ADD 2012.2.10
	if (fp != NULL) {									// ADD 2012.2.10
		fclose(fp);										// ADD 2012.2.10
	}													// ADD 2012.2.10
}

- (BOOL)applicationShouldTerminateAfterLastWindowClosed:(NSApplication *)theApplication
{
	return(YES);
}

@end
