//
//  LinerGauge.m
//  HA7 logger mini
//  Copyright(C)2012 Toyohiko Togashi tog001@nifty.com
//
//
//  This program is free software; you can redistribute it and/or modify it under the terms of the
//  GNU General Public License as published by the Free Software Foundation; either version 3
//  of the License, or (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY;
//  without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//  See the GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License along with this program.
//  If not, see <http://www.gnu.org/licenses/>
//
//
//    Update history
//    ---------- ----- -------------------------------------------
//    2012.02.08 v0.0  First cording
//    2012.02.12 v0.2  Commit
//

#import "LinerGauge.h"

@implementation LinerGauge
@synthesize val;
@synthesize val_l;
@synthesize val_h;
@synthesize val_w;
@synthesize val_c;
@synthesize val_peak1;
@synthesize val_peak2;
@synthesize val_low;
@synthesize val_high;
@synthesize val_warning;
@synthesize val_critical;

- (void)awakeFromNib {
	val_h = [val_high floatValue];
	val_l = [val_low floatValue];
	val_w = [val_warning floatValue];
	val_c = [val_critical floatValue];
	val   = 0;
	val_peak1 = self.val;
	val_peak2 = self.val;
}

- (void)drawRect:(NSRect)rect {
	float cx;
	float w;
	float h;
	float x1,y1;
	float x2,y2;
	float ratio;
	float zero;
	
	cx = NSMidX(rect);
	w  = NSWidth(rect);
	h  = NSHeight(rect);

	// Maximum peak line
	ratio = (val_peak1 - val_l) / (val_h - val_l);
	x1 = 0;
	y1 = h * ratio;
	x2 = w;
	y2 = y1;
	[[NSColor greenColor] set];
	[NSBezierPath setLineWidth:2.0];
	[NSBezierPath strokeLineFromPoint:NSMakePoint(x1,y1) toPoint:NSMakePoint(x2,y2)];
	
	// Minimum peak line
	ratio = (val_peak2 - val_l) / (val_h - val_l);
	x1 = 0;
	y1 = h * ratio;
	x2 = w;
	y2 = y1;
	[[NSColor greenColor] set];
	[NSBezierPath setLineWidth:2.0];
	[NSBezierPath strokeLineFromPoint:NSMakePoint(x1,y1) toPoint:NSMakePoint(x2,y2)];
	
	// Main bar
	ratio = (val - val_l) / (val_h - val_l);
	zero = -val_l / (val_h - val_l);
	x1 = cx;
	y1 = h * zero;
	x2 = cx;
	y2 = h * ratio;
	if (val > val_c) {
		[[NSColor redColor] set];
	} else if (val > val_w) {
		[[NSColor yellowColor] set];
	} else {
		[[NSColor greenColor] set];
	}
	[NSBezierPath setLineWidth:w];
	[NSBezierPath strokeLineFromPoint:NSMakePoint(x1,y1)     toPoint:NSMakePoint(x2,y2)];
	[NSBezierPath strokeLineFromPoint:NSMakePoint(x1,y1-3.0) toPoint:NSMakePoint(x1,y1+3.0)];
}

- (IBAction)setValue:(id)sender {
    val = [sender floatValue];
    [self setNeedsDisplay:YES];

	if (val_peak1 < val) {
		val_peak1 = val;
	}
	if (val_peak2 > val) {
		val_peak2 = val;
	}
}

- (IBAction)setIntValue:(int)sender {
    val = sender;
    [self setNeedsDisplay:YES];

	if (val_peak1 < val) {
		val_peak1 = val;
	}
	if (val_peak2 > val) {
		val_peak2 = val;
	}
}

- (IBAction)setFloatValue:(float)sender {
    val = sender;
    [self setNeedsDisplay:YES];

	if (val_peak1 < val) {
		val_peak1 = val;
	}
	if (val_peak2 > val) {
		val_peak2 = val;
	}
}

- (IBAction)setPeakVale:(id)sender {
    val_peak1 = val;
    val_peak2 = val;
    [self setNeedsDisplay:YES];
}

- (void)mouseDown:(NSEvent *)theEvent {
    val_peak1 = val;
    val_peak2 = val;
    [self setNeedsDisplay:YES];	
}

@end
